import sys
from datetime import datetime
from datetime import timedelta

import covjson2pandas
import requests

collection = "observations"
base_url = f"https://api.dataplatform.knmi.nl/edr/collections/{collection}"
token = "YOUR TOKEN"
headers = {"Authorization": token}


def locations():
    r = requests.get(base_url + "/locations", headers=headers)
    r.raise_for_status()
    return r.json()


def location(location, params):
    r = requests.get(base_url + "/locations/" + location, params, headers=headers)
    r.raise_for_status()
    df = covjson2pandas.convert_covjson_to_dataframe(r.json())

    return df


def position(params):
    r = requests.get(base_url + "/position", params, headers=headers)
    r.raise_for_status()
    df = covjson2pandas.convert_covjson_to_dataframe(r.json())

    return df


def main():
    locs = locations()
    schiphol = [f for f in locs["features"] if f["properties"]["name"] == "AMSTERDAM/SCHIPHOL AP"][0]

    now = datetime.now().isoformat("T", timespec="seconds")
    before = (datetime.now() - timedelta(days=7)).isoformat("T", timespec="seconds")

    params = {
        "coord": f"POINT({schiphol['geometry']['coordinates'][0]} {schiphol['geometry']['coordinates'][1]})",
        "datetime": f"{before}Z/{now}Z",
        "parameter-name": "t_dryb_10",
    }
    print(position(params))

    # Alternative
    # print(location(schiphol['properties']['location'], params))


if __name__ == "__main__":
    sys.exit(main())
