import pandas


def convert_covjson_to_dataframe(data):
    if data["type"] != "Coverage" or data["domain"]["domainType"] != "PointSeries":
        raise Exception("Unsupported CoverageJSON. Domain is not of type PointSeries")

    domain = data["domain"]
    parameters = data["parameters"]
    ranges = data["ranges"]

    if len(domain["axes"]["x"]["values"]) > 1 or len(domain["axes"]["y"]["values"]) > 1:
        raise Exception("Unsupported: Multiple stations")

    # Extract columns and timestamps
    columns = parameters.keys()
    index = domain["axes"]["t"]["values"]

    # Extract data
    values = {}
    for c in columns:
        values[c] = ranges[c]["values"]

    df = pandas.DataFrame(values, columns=columns, index=index)

    # Convert index to datetime
    df.index = pandas.to_datetime(df.index)

    return df
