#!/usr/bin/env python
# coding: utf-8

# # What kind of data is in the last 10 minute data?

# ## 1. Look up the Actuele 10 minuten data its metadata at https://dataplatform.knmi.nl/dataset/actuele10mindataknmistations-2
# ## 2. Get an Open Data API key at https://developer.dataplatform.knmi.nl/

# In[1]:


import logging

logging.basicConfig()
logger = logging.getLogger(__name__)
# logger.setLevel("DEBUG")
logger.setLevel("INFO")


# ## Set config for the variables

# In[2]:


api_url = "https://api.dataplatform.knmi.nl/open-data"
api_version = "v1"
api_key = "eyJvcmciOiI1ZTU1NGUxOTI3NGE5NjAwMDEyYTNlYjEiLCJpZCI6IjkxNDdjYTJhOTE2YTRjN2ZiZjI3YjExNWZhMTRkNTU3IiwiaCI6Im11cm11cjEyOCJ9"

dataset_name = "Actuele10mindataKNMIstations"
dataset_version = "2"
max_keys = 30


# ## 3. Try to retrieve a list of files to determine the last file

# In[3]:


import json
import requests
from datetime import datetime
from datetime import timedelta

# datetime now -> last whole hour; -2 hours since it takes file after the passed time.
start_after = (datetime.now() - timedelta(hours=2)).strftime("%Y%m%d%H00")
logger.debug(f"Retrieve file after: {start_after}")
start_after_filename_prefix = f"KMDS__OPER_P___10M_OBS_L2_{start_after}.nc"


# In[4]:


list_files_response = requests.get(
    f"{api_url}/{api_version}/datasets/{dataset_name}/versions/{dataset_version}/files",
    headers={"Authorization": api_key},
    params={"maxKeys": max_keys, "startAfterFilename": start_after_filename_prefix},
)
list_files = list_files_response.json()
logger.info(json.dumps(list_files, indent=4))


# ## 4. Download the last file

# In[5]:


file_name = list_files["files"][-1]["filename"]
logger.debug(file_name)

endpoint = f"{api_url}/{api_version}/datasets/{dataset_name}/versions/{dataset_version}/files/{file_name}/url"


# In[6]:


def download_file_from_temporary_download_url(download_url, file_name):
    try:
        with requests.get(download_url, stream=True) as r:
            r.raise_for_status()
            with open(file_name, "wb") as f:
                counter = 0
                for chunk in r.iter_content(chunk_size=8192):
                    counter += 1
                    logger.debug(f"write chunk {counter}")
                    f.write(chunk)
    except Exception:
        logger.exception("Unable to download file using download URL")
        sys.exit(1)

    logger.info(f"Successfully downloaded dataset file to {file_name}")


get_file_response = requests.get(endpoint, headers={"Authorization": api_key})
if get_file_response.status_code != 200:
    logger.error("Unable to retrieve download url for file")
    logger.error(get_file_response.text)
    sys.exit(1)

download_url = get_file_response.json().get("temporaryDownloadUrl")
download_file_from_temporary_download_url(download_url, f"./download/{file_name}")


# ## Open multiple NetCDF files

# In[7]:


import xarray
import os

ds = xarray.open_mfdataset(os.path.join("/home/vervoort/Desktop/workshop/dataplatform-examples-external/open-data-api/data", "*.nc"))
# ds = xarray.open_dataset(file_name)
df = ds.to_dataframe()


# In[8]:


# general_info = {key: {"type": value.dtype, "Attributes": value.attrs} for key, value in ds.variables.items()}
general_info = {key: value.attrs.get("long_name") for key, value in ds.variables.items()}
logger.info(json.dumps(general_info, indent=4))


# ## 5. Exploratory Data Analysis (EDA)

# In[9]:


new_columns = ["stationname", "lat", "lon", "height", "ss", "qg", "nc", "R24H", "dr", "rg", "pr"]


# In[10]:


column_description = {key: value for key, value in general_info.items() if key in new_columns}
logger.info(json.dumps(column_description, indent=4))


# In[11]:


subset = df[new_columns]


# In[12]:


from pandas_profiling import ProfileReport

profile = ProfileReport(subset, title="Pandas Profiling Report", explorative=True)


# In[13]:


profile


# ## 6. Analyse the data

# In[14]:


import matplotlib.pyplot as plt

plot_cols = ["ss", "qg", "nc", "R24H"]

df_plot = subset.loc[subset['stationname'] == "VOLKEL"]
df_plot = df_plot[plot_cols].reset_index()
df_plot = df_plot.rename(columns=column_description)


# In[15]:


df_plot.plot(x="time", subplots=True, layout=(1, 4), figsize=(20,4), rot=45, sharex=False, sharey=False, title="Station Vonkel")
plt.show()


# ## Plot the stations on a map

# In[16]:


# import plotly.express as px
# import geopandas as gpd

# geo_df = gpd.read_file(gpd.datasets.get_path('naturalearth_cities'))

# px.set_mapbox_access_token(open(".mapbox_token").read())
# fig = px.scatter_geo(geo_df,
#                     lat=geo_df.geometry.y,
#                     lon=geo_df.geometry.x,
#                     hover_name="name")
# fig.show()


# In[ ]:




